@@ -3,19 +3,19 @@ from os.path import abspath, dirname, join
 from pathlib import Path
 from typing import IO, TYPE_CHECKING, Any, BinaryIO, Dict, Iterable, List, Optional, Tuple, Union
 
-from appdirs import user_cache_dir
 from dateutil.relativedelta import relativedelta
+from platformdirs import user_cache_dir
 
 # iNaturalist URLs
 API_V0_BASE_URL = 'https://www.inaturalist.org'
 API_V1_BASE_URL = 'https://api.inaturalist.org/v1'
 API_V2_BASE_URL = 'https://api.inaturalist.org/v2'
-DWC_ARCHIVE_URL = 'http://www.inaturalist.org/observations/gbif-observations-dwca.zip'
 EXPORT_URL = 'https://www.inaturalist.org/observations/export'
 INAT_BASE_URL = API_V0_BASE_URL
 INAT_REPO = 'https://raw.githubusercontent.com/inaturalist/inaturalist/main'
 ICONIC_TAXA_BASE_URL = f'{INAT_REPO}/app/assets/images/iconic_taxa'
 PHOTO_BASE_URL = 'https://static.inaturalist.org/photos'
+PHOTO_CC_BASE_URL = 'https://inaturalist-open-data.s3.amazonaws.com/photos'
 PHOTO_INFO_BASE_URL = 'https://www.inaturalist.org/photos'
 
 # Prefix used for keyring entries
@@ -26,6 +26,7 @@ PER_PAGE_RESULTS = 200  # Default number of records per page for paginated queri
 LARGE_REQUEST_WARNING = 5000  # Show a warning for queries that will return over this many results
 
 # Rate-limiting and retry settings
+CONNECT_TIMEOUT = 5
 MAX_DELAY = 60  # Maximum time to wait for rate-limiting before aborting
 REQUEST_BURST_RATE = 5
 REQUESTS_PER_SECOND = 1
@@ -48,15 +49,21 @@ SAMPLE_DATA_DIR = join(PROJECT_DIR, 'test', 'sample_data')
 
 # Cache settings
 CACHE_EXPIRATION = {
-    'api.inaturalist.org/v*/controlled_terms*': timedelta(days=1),
-    'api.inaturalist.org/v*/places*': timedelta(days=1),
-    'api.inaturalist.org/v*/taxa*': timedelta(days=1),
+    'api.inaturalist.org/*autocomplete': timedelta(days=30),
+    'api.inaturalist.org/v*/controlled_terms*': timedelta(days=7),
+    'api.inaturalist.org/v*/places*': timedelta(days=7),
+    'api.inaturalist.org/v*/taxa*': timedelta(days=7),
+    f'{PHOTO_CC_BASE_URL}/*': -1,
+    f'{PHOTO_BASE_URL}/*': -1,
+    f'{ICONIC_TAXA_BASE_URL}/*': -1,
     '*': timedelta(minutes=30),
 }
 CACHE_FILE = join(CACHE_DIR, 'api_requests.db')
+RATELIMIT_FILE = join(CACHE_DIR, 'api_ratelimit.db')
 TOKEN_EXPIRATION = timedelta(hours=1)
+JWT_EXPIRATION = timedelta(days=1)
 
-# Response formats supported by GET /observations endpoint
+# Response formats supported by v0 GET /observations endpoint
 OBSERVATION_FORMATS = ['atom', 'csv', 'dwc', 'json', 'kml', 'widget']
 
 # IUCN Conservation status codes; for more info, see: https://www.iucnredlist.org
@@ -136,6 +143,7 @@ GEOPRIVACY_LEVELS = ['obscured', 'obscured_private', 'open', 'private']
 HAS_PROPERTIES = ['photo', 'geo']
 HISTOGRAM_DATE_FIELDS = ['created', 'observed']
 HISTOGRAM_INTERVALS = ['year', 'month', 'week', 'day', 'hour', 'month_of_year', 'week_of_year']
+ICON_SIZES = {'icon': 32, 'square': 75, 'small': 75, 'medium': 200, 'large': 200, 'original': 200}
 ID_CATEGORIES = ['improving', 'supporting', 'leading', 'maverick']
 INBOXES = ['inbox', 'sent', 'any']
 ORDER_DIRECTIONS = ['asc', 'desc']
@@ -199,6 +207,8 @@ DATETIME_PARAMS = [
     'observed_on_string',
     'older_than',
     'on',
+    'prefers_rule_d1',
+    'prefers_rule_d2',
     'since',
     'updated_since',
 ]
