@@ -1,11 +1,11 @@
-"""Type conversion utilities"""
+"""Type conversion utilities used for both requests and responses"""
 import re
 from datetime import date, datetime, timedelta
 from io import BytesIO
 from logging import getLogger
 from os.path import abspath, expanduser
 from pathlib import Path
-from typing import IO, Any, Dict, List, MutableSequence, Optional, Union
+from typing import IO, Any, Dict, List, Mapping, MutableSequence, Optional, Union
 from warnings import catch_warnings, simplefilter
 
 from dateutil.parser import UnknownTimezoneWarning  # type: ignore  # (missing from type stubs)
@@ -71,17 +71,19 @@ def convert_all_timestamps(results: List[ResponseResult]) -> List[ResponseResult
     return results
 
 
-def convert_histogram(response: JsonResponse) -> HistogramResponse:
+def convert_observation_histogram(response: JsonResponse) -> HistogramResponse:
     """Convert a response containing time series data into a single ``{date: value}`` dict"""
     # The inner result object's key will be the name of the interval requested
     interval = next(iter(response['results'].keys()))
-    histogram = response['results'][interval]
+    return convert_histogram(response['results'][interval], interval)
 
-    # Convert keys to appropriate type depending on interval
+
+def convert_histogram(result: JsonResponse, interval: str):
+    """Convert keys to appropriate type depending on interval"""
     if interval in ['month_of_year', 'week_of_year']:
-        return {int(k): v for k, v in histogram.items()}
+        return {int(k): v for k, v in result.items()}
     else:
-        return {parse_date(k): v for k, v in histogram.items()}
+        return {parse_date(k): v for k, v in result.items()}
 
 
 # Type conversion functions for individual results or values
@@ -90,7 +92,7 @@ def convert_histogram(response: JsonResponse) -> HistogramResponse:
 
 def convert_csv_list(obj: Any) -> str:
     """Convert list parameters into an API-compatible (comma-delimited) string"""
-    if isinstance(obj, list):
+    if isinstance(obj, list) or isinstance(obj, tuple):
         return ','.join(map(str, obj))
     else:
         return obj
@@ -268,7 +270,7 @@ def strip_empty_values(values: Dict) -> Dict:
 
 
 def ensure_file_obj(value: AnyFile, session: Session = None) -> IO:
-    """Given a file path or URL, load data into a file-like object"""
+    """Load data into a file-like object, if it isn't already. Accepts local file paths and URLs."""
     # Load from URL
     if isinstance(value, str) and URL_PATTERN.match(value):
         session = session or Session()
@@ -312,9 +314,10 @@ def format_dimensions(dimensions: Union[Dict[str, int], Dimensions, None]) -> Di
     """Simplify a 'dimensions' dict into a ``(width, height)`` tuple"""
     if not dimensions:
         return (0, 0)
-    if isinstance(dimensions, tuple):
-        return dimensions
-    return dimensions.get("width", 0), dimensions.get("height", 0)
+    if isinstance(dimensions, (tuple, list, set)):
+        return (dimensions[0], dimensions[1])
+    elif isinstance(dimensions, Mapping):
+        return dimensions.get('width', 0), dimensions.get('height', 0)
 
 
 def format_file_size(n_bytes: int) -> str:
