@@ -7,12 +7,15 @@ These functions will accept any of the following:
 * A list of response objects
 * A single response object
 """
-from copy import deepcopy
-from datetime import date, datetime
+import json
+from datetime import date, datetime, timedelta
 from functools import partial
-from typing import List, Type
+from logging import basicConfig, getLogger
+from typing import List, Mapping, Type
 
-from pyinaturalist.constants import ResponseOrResults, ResponseResult
+from requests import PreparedRequest, Response
+
+from pyinaturalist.constants import DATETIME_SHORT_FORMAT, ResponseOrResults, ResponseResult
 from pyinaturalist.converters import ensure_list
 from pyinaturalist.models import (
     Annotation,
@@ -20,9 +23,12 @@ from pyinaturalist.models import (
     BaseModelCollection,
     Comment,
     ControlledTerm,
+    ControlledTermCount,
     ControlledTermValue,
     Identification,
     LifeList,
+    ListedTaxon,
+    Message,
     Observation,
     ObservationField,
     ObservationFieldValue,
@@ -33,47 +39,70 @@ from pyinaturalist.models import (
     SearchResult,
     Taxon,
     TaxonCount,
+    TaxonSummary,
     User,
-    get_model_fields,
 )
+from pyinaturalist.paginator import Paginator
 
-# If rich is installed, update its pretty-printer to include model properties
-try:
-    from rich import pretty, print
 
-    pretty._get_attr_fields = get_model_fields
-    pretty.install()
-except ImportError:
-    pass
+def enable_logging(level: str = 'INFO', external_level: str = 'WARNING'):
+    """Configure logging to standard output with prettier tracebacks, formatting, and terminal
+    colors (if supported).
+
+    If you prefer, logging can be configured with the stdlib ``logging`` module instead; this just
+    provides some convenient defaults.
+
+    Args:
+        level: Logging level to use for pyinaturalist
+        external_level: Logging level to use for other libraries
+    """
+    from rich.logging import RichHandler
+
+    basicConfig(
+        format='%(message)s',
+        datefmt='[%m-%d %H:%M:%S]',
+        handlers=[RichHandler(rich_tracebacks=True, markup=True)],
+        level=external_level,
+    )
+    getLogger('pyinaturalist').setLevel(level)
+    getLogger('pyinaturalist_convert').setLevel(level)
 
 
 # Default colors for table headers
 HEADER_COLORS = {
     'Category': 'violet',
     'Comment': 'green',
+    'Comments': 'blue',
     'Common name': 'blue',
     'Count': 'blue',
     'Created at': 'blue',
+    'Date': 'blue',
     'Description': 'green',
     'Dimensions': 'blue',
     'Display name': 'violet',
+    'Establishment means': 'violet',
+    'From': 'magenta',
     'From CV': 'white',
-    'ID count': 'blue',
+    'Identifications': 'blue',
     'ID': 'cyan',
     'Label': 'green',
     'Latitude': 'blue',
     'License': 'green',
+    'Life list': 'green',
     'Location': 'white',
     'Longitude': 'blue',
     'Name': 'magenta',
-    'Obs. count': 'blue',
+    'Observations': 'blue',
     'Observed on': 'blue',
+    'Place ID': 'cyan',
     'Rank': 'violet',
     'Scientific name': 'green',
     'Score': 'green',
+    'Subject': 'green',
     'Taxon ID': 'cyan',
     'Taxon': 'green',
     'Title': 'green',
+    'To': 'magenta',
     'Type': 'blue',
     'URL': 'white',
     'User': 'magenta',
@@ -86,8 +115,10 @@ HEADER_COLORS = {
 UNIQUE_RESPONSE_ATTRS = {
     'vote_score': Annotation,
     'disagreement': Identification,
+    'thread_id': Message,
     'body': Comment,  # Subset of ID attrs; if it's not an ID, assume it's a comment
     'multivalued': ControlledTerm,
+    'controlled_value': ControlledTermCount,
     'blocking': ControlledTermValue,
     'count_without_taxon': LifeList,
     'captive': Observation,
@@ -98,8 +129,10 @@ UNIQUE_RESPONSE_ATTRS = {
     'standard': Place,
     'project_type': Project,
     'score': SearchResult,
-    'rank': Taxon,
+    'last_observation_id': ListedTaxon,
+    'listed_taxon': TaxonSummary,
     'count': TaxonCount,
+    'rank': Taxon,
     'roles': User,
 }
 
@@ -109,7 +142,10 @@ def pprint(values: ResponseOrObjects):
 
     **Experimental:** May also be used on most raw JSON API responses
     """
-    print(format_table(values))
+    try:
+        print(format_table(values))
+    except ValueError:
+        print(values)
 
 
 def detect_type(value: ResponseResult) -> Type[BaseModel]:
@@ -125,6 +161,9 @@ def ensure_model_list(values: ResponseOrObjects) -> List[BaseModel]:
     """If the given values are raw JSON responses, attempt to detect their type and convert to
     model objects
     """
+    if isinstance(values, Paginator):
+        return values.all()
+
     values = ensure_list(values)
     if isinstance(values, BaseModelCollection) or isinstance(values[0], BaseModel):
         return values  # type: ignore
@@ -152,7 +191,7 @@ def format_table(values: ResponseOrObjects):
     # Display any date/datetime values in short format
     def _str(value):
         if isinstance(value, (date, datetime)):
-            return value.strftime('%b %d, %Y')
+            return value.strftime(DATETIME_SHORT_FORMAT)
         return str(value) if value is not None else ''
 
     columns = [Column(header, style=style) for header, style in headers.items()]
@@ -163,34 +202,56 @@ def format_table(values: ResponseOrObjects):
     return table
 
 
-# TODO: This maybe belongs in a different module
-def simplify_observations(observations: ResponseOrResults, align: bool = False) -> List[ResponseResult]:
-    """Flatten out some nested data structures within observation records:
-
-    * annotations
-    * comments
-    * identifications
-    * non-owner IDs
-    """
-    return [_simplify_observation(o) for o in ensure_list(observations)]
+def format_request(request: PreparedRequest, dry_run: bool = False) -> str:
+    """Format HTTP request info"""
+    headers = _format_headers(request.headers)
+    body = _format_body(request.body)
+    dry_run_str = ' (DRY RUN) ' if dry_run else ''
+    return f'Request:{dry_run_str}\n{request.method} {request.url}\n{headers}\n{body}'
 
 
-def _simplify_observation(obs):
-    # Reduce annotations to IDs and values
-    obs = deepcopy(obs)
-    obs['annotations'] = [
-        (a['controlled_attribute_id'], a['controlled_value_id']) for a in obs['annotations']
+def format_response(response: Response) -> str:
+    """Format HTTP response info, including whether it came from the cache"""
+    headers = _format_headers(response.headers)
+    error_msg = f' {response.text}' if not response.ok else ''
+
+    def _expires_str():
+        if response.expires:
+            expires_delta = response.expires - datetime.utcnow()
+            expires_delta -= timedelta(microseconds=expires_delta.microseconds)
+            return f'expires in {expires_delta}'
+        else:
+            return 'never expires'
+
+    cached = f'cached; {_expires_str()}' if getattr(response, 'from_cache', False) else 'not cached'
+    return f'Response ({cached}):\n{response.status_code} {response.reason}{error_msg}\n{headers}'
+
+
+def _format_headers(headers: Mapping[str, str]) -> str:
+    ignore_headers = [
+        'Access-Control-Allow-Headers',
+        'Access-Control-Allow-Methods',
+        'Access-Control-Allow-Origin',
+        'X-Content-Type-Options',
     ]
+    headers_dict = {k: v for k, v in headers.items() if k not in ignore_headers}
+    if 'Authorization' in headers_dict:
+        headers_dict['Authorization'] = '[REDACTED]'
 
-    # Reduce identifications to just a list of identification IDs and taxon IDs
-    obs['identifications'] = [(i['id'], i['taxon_id']) for i in obs['identifications']]
-    obs['non_owner_ids'] = [(i['id'], i['taxon_id']) for i in obs['non_owner_ids']]
+    return '\n'.join([f'{k}: {v}' for k, v in headers_dict.items()])
 
-    # Reduce comments to usernames and comment text
-    obs['comments'] = [(c['user']['login'], c['body']) for c in obs['comments']]
-    del obs['observation_photos']
 
-    return obs
+def _format_body(body):
+    if not body:
+        return ''
+    try:
+        body = json.loads(body)
+        for key in ['password', 'client_secret']:
+            if key in body:
+                body[key] = '[REDACTED]'
+        return body
+    except Exception:
+        return '(non-JSON request body)'
 
 
 def _format_model_objects(obj: ResponseOrResults, cls: Type[BaseModel], **kwargs):
@@ -208,3 +269,11 @@ format_search_results = partial(_format_model_objects, cls=SearchResult)
 format_species_counts = partial(_format_model_objects, cls=TaxonCount)
 format_taxa = partial(_format_model_objects, cls=Taxon)
 format_users = partial(_format_model_objects, cls=User)
+
+# If rich is installed, install pretty-printer
+try:
+    from rich import pretty, print
+
+    pretty.install()
+except ImportError:
+    pass
