@@ -1,6 +1,6 @@
 from datetime import datetime
 
-from pyinaturalist.constants import ID_CATEGORIES, TableRow
+from pyinaturalist.constants import DATETIME_SHORT_FORMAT, ID_CATEGORIES, TableRow
 from pyinaturalist.models import (
     BaseModel,
     LazyProperty,
@@ -29,7 +29,9 @@ class Identification(BaseModel):
         default=None, doc='Indicates if this identification disagrees with previous ones'
     )
     hidden: bool = field(default=None)
-    own_observation: bool = field(default=None, doc='Indicates if the indentifier is also the observer')
+    own_observation: bool = field(
+        default=None, doc='Indicates if the indentifier is also the observer'
+    )
     previous_observation_taxon_id: int = field(default=None, doc='Previous observation taxon ID')
     taxon_change: bool = field(default=None)  # TODO: confirm type
     taxon_id: int = field(default=None, doc='Identification taxon ID')
@@ -38,7 +40,9 @@ class Identification(BaseModel):
         default=None, doc='Indicates if the taxon was selected from computer vision results'
     )
     taxon: property = LazyProperty(Taxon.from_json, type=Taxon, doc='Identification taxon')
-    user: property = LazyProperty(User.from_json, type=User, doc='User that added the indentification')
+    user: property = LazyProperty(
+        User.from_json, type=User, doc='User that added the indentification'
+    )
 
     # Unused attributes
     # created_at_details: {}
@@ -61,6 +65,7 @@ class Identification(BaseModel):
     def __str__(self) -> str:
         """Format into a condensed summary: id, what, when, and who"""
         return (
-            f'[{self.id}] {self.taxon.full_name} ({self.category}) added on {self.created_at} '
+            f'[{self.id}] {self.taxon.emoji} {self.taxon.full_name} ({self.category}) '
+            f'added on {self.created_at.strftime(DATETIME_SHORT_FORMAT)} '
             f'by {self.user.login}'
         )
