@@ -8,6 +8,7 @@ from pyinaturalist.constants import (
     ALL_LICENSES,
     DATETIME_SHORT_FORMAT,
     GEOPRIVACY_LEVELS,
+    INAT_BASE_URL,
     QUALITY_GRADES,
     Coordinates,
     DateTime,
@@ -91,9 +92,10 @@ class Observation(BaseModel):
     owners_identification_from_vision: bool = field(
         default=None, doc="Indicates if the owner's ID was selected from computer vision results"
     )
-
     place_guess: str = field(default=None, doc='Place name determined from observation coordinates')
-    place_ids: List[int] = field(factory=list)
+    place_ids: List[int] = field(
+        factory=list, doc='Place IDs associated with the observation coordinates'
+    )
     positional_accuracy: int = field(
         default=None, doc='GPS accuracy in meters (real accuracy, if obscured)'
     )
@@ -101,6 +103,15 @@ class Observation(BaseModel):
         factory=dict,
         doc='Any user observation preferences (related to community IDs, coordinate access, etc.)',
     )
+    private_location: Coordinates = coordinate_pair(
+        doc=':fa:`lock` Private location in ``(latitude, logitude)`` decimal degrees'
+    )
+    private_place_ids: List[int] = field(
+        factory=list, doc=':fa:`lock` Place IDs associated with the private observation coordinates'
+    )
+    private_place_guess: str = field(
+        default=None, doc=':fa:`lock` Place name determined from private observation coordinates'
+    )
     project_ids: List[int] = field(factory=list, doc='All associated project IDs')
     project_ids_with_curator_id: List[int] = field(
         factory=list, doc='Project IDs with a curator identification for this observation'
@@ -187,12 +198,17 @@ class Observation(BaseModel):
     # Convert observation timestamps prior to __attrs_init__
     def __init__(
         self,
-        created_at_details: Dict = None,
-        observed_on_string: str = None,
-        observed_time_zone: str = None,
-        time_zone_offset: str = None,
+        # created_at_details: Dict = None,
+        # observed_on_string: str = None,
+        # observed_time_zone: str = None,
+        # time_zone_offset: str = None,
         **kwargs,
     ):
+        created_at_details = kwargs.pop('created_at_details', None)
+        observed_on_string = kwargs.pop('observed_on_string', None)
+        observed_time_zone = kwargs.pop('observed_time_zone', None)
+        time_zone_offset = kwargs.pop('time_zone_offset', None)
+
         tz_offset = time_zone_offset
         tz_name = observed_time_zone
         created_date = (created_at_details or {}).get('date')
@@ -204,6 +220,9 @@ class Observation(BaseModel):
                 observed_on_string, tz_offset, tz_name, ignoretz=True
             )
 
+        if not kwargs.get('uri'):
+            kwargs['uri'] = f'{INAT_BASE_URL}/observations/{kwargs.get("id", "")}'
+
         self.__attrs_init__(**kwargs)  # type: ignore
 
     @classmethod
@@ -241,7 +260,7 @@ class Observation(BaseModel):
 
     def __str__(self) -> str:
         return (
-            f'[{self.id}] {self.taxon.full_name} '
+            f'[{self.id}] {self.taxon.emoji} {self.taxon.full_name} '
             f'observed on {self.observed_on.strftime(DATETIME_SHORT_FORMAT)} '
             f'by {self.user.login} at {self.place_guess or self.location}'
         )
