@@ -1,16 +1,25 @@
-from io import BytesIO
 from typing import BinaryIO, Optional, Tuple
 
 import requests
 
-from pyinaturalist.constants import ALL_LICENSES, CC_LICENSES, PHOTO_INFO_BASE_URL, PHOTO_SIZES, TableRow
+from pyinaturalist.constants import (
+    ALL_LICENSES,
+    CC_LICENSES,
+    ICON_SIZES,
+    ICONIC_TAXA_BASE_URL,
+    PHOTO_BASE_URL,
+    PHOTO_CC_BASE_URL,
+    PHOTO_INFO_BASE_URL,
+    PHOTO_SIZES,
+    TableRow,
+)
 from pyinaturalist.converters import format_dimensions, format_license
 from pyinaturalist.models import BaseModel, define_model, field
 
 
 @define_model
 class Photo(BaseModel):
-    """📷 An observation photo, based on the schema of photos from:
+    """:fa:`camera` An observation photo, based on the schema of photos from:
 
     * `GET /observations <https://api.inaturalist.org/v1/docs/#!/Observations/get_observations>`_
     * `GET /taxa <https://api.inaturalist.org/v1/docs/#!/Taxa/get_taxa>`
@@ -18,7 +27,10 @@ class Photo(BaseModel):
 
     attribution: str = field(default=None, doc='License attribution')
     license_code: str = field(
-        default=None, converter=format_license, options=ALL_LICENSES, doc='Creative Commons license code'
+        default=None,
+        converter=format_license,
+        options=ALL_LICENSES,
+        doc='Creative Commons license code',
     )
     original_dimensions: Tuple[int, int] = field(
         converter=format_dimensions, default=(0, 0), doc='Dimensions of original image'
@@ -30,14 +42,23 @@ class Photo(BaseModel):
     # flags: List = field(factory=list)
 
     def __attrs_post_init__(self):
-        if not self.url:
-            return
+        # If there's no URL, make a guess based on ID and license:
+        self.url = self.url or (
+            f'{PHOTO_CC_BASE_URL}/{self.id}/original.jpg'
+            if self.has_cc_license
+            else f'{PHOTO_BASE_URL}/{self.id}?size=original'
+        )
 
         # Get a URL format string to get different photo sizes. Note: default URL may be any size.
         for size in PHOTO_SIZES:
             if f'{size}.' in self.url:
                 self._url_format = self.url.replace(size, '{size}')
 
+    @property
+    def ext(self) -> str:
+        """File extension from URL"""
+        return self.url.lower().split('.')[-1].split('?')[0]
+
     @property
     def dimensions_str(self) -> str:
         """Dimensions as a string, formatted as ``{width}x{height}``"""
@@ -63,6 +84,11 @@ class Photo(BaseModel):
         """Image URL (medium size)"""
         return self.url_size('medium')
 
+    @property
+    def mimetype(self) -> str:
+        """MIME type of the image"""
+        return f'image/{self.ext.replace("jpg", "jpeg")}'
+
     @property
     def original_url(self) -> Optional[str]:
         """Image URL (original size)"""
@@ -91,13 +117,11 @@ class Photo(BaseModel):
 
     def open(self, size: str = 'large') -> BinaryIO:
         """Download the image and return as a file-like object"""
-        url = self.url_size(size)
-        return requests.get(url, stream=True).raw if url else BytesIO()
+        url = self.url_size(size) or self.url
+        return requests.get(url, stream=True).raw
 
     def show(self, size: str = 'large'):
-        """Download and display the image with the system's default image viewer.
-        Requires ``pillow``.
-        """
+        """Display the image with the system's default image viewer. Requires ``pillow``."""
         from PIL import Image
 
         img = Image.open(self.open(size=size))
@@ -114,3 +138,28 @@ class Photo(BaseModel):
 
     def __str__(self) -> str:
         return f'[{self.id}] {self.original_url} ({self.license_code}, {self.dimensions_str})'
+
+
+@define_model
+class IconPhoto(Photo):
+    """:fa:`camera` Class used for displaying an iconic taxon in place of a taxon photo"""
+
+    def __attrs_post_init__(self):
+        self._url_format = self.url.replace('.png', '-{size}px.png')
+
+    @classmethod
+    def from_iconic_taxon(cls, iconic_taxon_name: str):
+        url = f'{ICONIC_TAXA_BASE_URL}/{iconic_taxon_name.lower()}.png'
+        return cls(url=url)  # type: ignore  # A weird false positive as of mypy 0.950
+
+    @property
+    def icon_url(self) -> Optional[str]:
+        """Image URL (32px icon size)"""
+        return self.url_size('icon')
+
+    def url_size(self, size: str) -> str:
+        size = size.replace('thumbnail', 'square').replace('thumb', 'square')
+        return self._url_format.format(size=ICON_SIZES.get(size, 'square'))
+
+    def __str__(self) -> str:
+        return self.url
