@@ -4,7 +4,7 @@ from attr import fields_dict
 
 from pyinaturalist.constants import (
     ICONIC_EMOJI,
-    ICONIC_TAXA_BASE_URL,
+    ICONIC_TAXA,
     INAT_BASE_URL,
     RANKS,
     DateTime,
@@ -25,6 +25,7 @@ from pyinaturalist.models import (
     define_model_collection,
     field,
 )
+from pyinaturalist.models.photo import IconPhoto
 
 
 @define_model
@@ -55,7 +56,7 @@ class Taxon(BaseModel):
         default=0, doc='ID of the iconic taxon (e.g., general taxon "category")'
     )
     iconic_taxon_name: str = field(
-        default='unknown', doc='Name of the iconic taxon (e.g., general taxon "category")'
+        default=None, doc='Name of the iconic taxon (e.g., general taxon "category")'
     )
     is_active: bool = field(
         default=None, doc='Indicates if the taxon is active (and not renamed, moved, etc.)'
@@ -134,6 +135,12 @@ class Taxon(BaseModel):
     # photos_locked: bool = field(default=None)
     # universal_search_rank: int = field(default=None)
 
+    def __attrs_post_init__(self):
+        if not self.iconic_taxon_name:
+            self.iconic_taxon_name = ICONIC_TAXA.get(self.iconic_taxon_id, 'Unknown')
+        if not self.default_photo:
+            self.default_photo = self.icon
+
     @classmethod
     def from_sorted_json_list(cls, value: JsonResponse) -> List['Taxon']:
         """Sort Taxon objects by rank then by name"""
@@ -165,16 +172,22 @@ class Taxon(BaseModel):
         """Taxon rank, scientific name, common name (if available), and emoji"""
         if not self.name and not self.rank:
             return 'unknown taxon'
-        if not self.name:
+        elif not self.name:
             return f'{self.rank.title()}: {self.id}'
+        elif not self.rank:
+            return self.name
 
         common_name = f' ({self.preferred_common_name})' if self.preferred_common_name else ''
-        return f'{self.emoji} {self.rank.title()}: {self.name}{common_name}'
+        return f'{self.rank.title()}: {self.name}{common_name}'
+
+    @property
+    def icon(self) -> IconPhoto:
+        return IconPhoto.from_iconic_taxon(self.iconic_taxon_name)
 
     @property
     def icon_url(self) -> str:
-        """URL for the iconic taxon's icon"""
-        return f'{ICONIC_TAXA_BASE_URL}/{self.iconic_taxon_name.lower()}-75px.png'
+        """Iconic URL for the icon of the iconic taxon"""
+        return str(self.icon.thumbnail_url)
 
     @property
     def gbif_url(self) -> str:
@@ -186,11 +199,17 @@ class Taxon(BaseModel):
         """Immediate parent, if any"""
         return self.ancestors[-1] if self.ancestors else None
 
+    @property
+    def taxonomy(self) -> Dict[str, str]:
+        """Ancestor + current taxon as a ``{rank: name}`` dict"""
+        return {t.rank: t.name for t in self.ancestors + [self]}
+
     @property
     def url(self) -> str:
         """Info URL on iNaturalist.org"""
         return f'{INAT_BASE_URL}/taxa/{self.id}'
 
+    # TODO: Probably remove this
     @classmethod
     def from_id(cls, id: int) -> 'Taxon':
         """Lookup and create a new Taxon object by ID"""
@@ -216,7 +235,7 @@ class Taxon(BaseModel):
         }
 
     def __str__(self) -> str:
-        return f'[{self.id}] {self.full_name}' if self.name else self.full_name
+        return f'[{self.id}] {self.emoji} {self.full_name}'
 
 
 @define_model
@@ -248,7 +267,7 @@ class TaxonCount(Taxon):
         }
 
     def __str__(self) -> str:
-        return f'[{self.id}] {self.full_name}: {self.count}'
+        return f'[{self.id}] {self.emoji} {self.full_name}: {self.count}'
 
 
 @define_model_collection
