@@ -2,7 +2,7 @@
 import threading
 from logging import getLogger
 from os import getenv
-from typing import Dict
+from typing import Dict, Type
 from unittest.mock import Mock
 
 import forge
@@ -10,14 +10,23 @@ from requests import PreparedRequest, Request, Response, Session
 from requests.adapters import HTTPAdapter
 from requests.utils import default_user_agent
 from requests_cache import CacheMixin, ExpirationTime
-from requests_ratelimiter import LimiterMixin
+from requests_ratelimiter import (
+    AbstractBucket,
+    BucketFullException,
+    Limiter,
+    LimiterMixin,
+    RequestRate,
+    SQLiteBucket,
+)
 from urllib3.util import Retry
 
 import pyinaturalist
 from pyinaturalist.constants import (
     CACHE_EXPIRATION,
     CACHE_FILE,
+    CONNECT_TIMEOUT,
     MAX_DELAY,
+    RATELIMIT_FILE,
     REQUEST_BURST_RATE,
     REQUEST_RETRIES,
     REQUEST_TIMEOUT,
@@ -31,7 +40,7 @@ from pyinaturalist.constants import (
     RequestParams,
 )
 from pyinaturalist.converters import ensure_file_obj
-from pyinaturalist.formatters import format_request
+from pyinaturalist.formatters import format_request, format_response
 from pyinaturalist.request_params import (
     convert_url_ids,
     preprocess_request_body,
@@ -42,6 +51,13 @@ from pyinaturalist.request_params import (
 MOCK_RESPONSE = Mock(spec=Response)
 MOCK_RESPONSE.json.return_value = {'results': [], 'total_results': 0, 'access_token': ''}
 
+# Rate limiter specific to forced refresh request
+REFRESH_LIMITER = Limiter(
+    RequestRate(1, 122),
+    bucket_class=SQLiteBucket,
+    bucket_kwargs={'path': RATELIMIT_FILE},
+)
+
 logger = getLogger('pyinaturalist')
 thread_local = threading.local()
 
@@ -68,6 +84,7 @@ class ClientSession(CacheMixin, LimiterMixin, Session):
         per_minute: int = REQUESTS_PER_MINUTE,
         per_day: float = REQUESTS_PER_DAY,
         burst: int = REQUEST_BURST_RATE,
+        bucket_class: Type[AbstractBucket] = SQLiteBucket,
         retries: int = REQUEST_RETRIES,
         backoff_factor: float = RETRY_BACKOFF,
         timeout: int = REQUEST_TIMEOUT,
@@ -84,6 +101,7 @@ class ClientSession(CacheMixin, LimiterMixin, Session):
             per_minute: Max requests per minute
             per_day: Max requests per day
             burst: Max number of consecutive requests allowed before applying per-second rate-limiting
+            bucket_class: Rate-limiting backend to use. Defaults to a persistent SQLite database.
             retries: Maximum number of times to retry a failed request
             backoff_factor: Factor for increasing delays between retries
             timeout: Maximum number of seconds to wait for a response from the server
@@ -97,16 +115,20 @@ class ClientSession(CacheMixin, LimiterMixin, Session):
             kwargs.setdefault('urls_expire_after', CACHE_EXPIRATION)
         self.timeout = timeout
 
-        # Initialize with caching and rate-limiting settings
         super().__init__(  # type: ignore  # false positive
+            # Cache settings
             cache_name=cache_file,
             backend='sqlite',
             expire_after=expire_after,
-            ignored_parameters=['access_token'],
+            ignored_parameters=['Authorization', 'access_token'],
             old_data_on_error=True,
+            # Rate limit settings
+            bucket_class=bucket_class,
+            bucket_kwargs={'path': RATELIMIT_FILE},
             per_second=per_second,
             per_minute=per_minute,
             per_day=per_day,
+            per_host=True,
             burst=burst,
             max_delay=MAX_DELAY,
             **kwargs,
@@ -126,23 +148,50 @@ class ClientSession(CacheMixin, LimiterMixin, Session):
         adapter = HTTPAdapter(max_retries=retry)
         self.mount('https://', adapter)
 
-    def send(self, request: PreparedRequest, **kwargs) -> Response:  # type: ignore  # false positive
-        """Send a request with caching, rate-limiting, and retries"""
-        kwargs.setdefault('timeout', self.timeout)
-        return super().send(request, **kwargs)
+    def send(  # type: ignore  # Adds kwargs not present in Session.send()
+        self,
+        request: PreparedRequest,
+        expire_after: ExpirationTime = None,
+        refresh: bool = False,
+        timeout: int = None,
+        **kwargs,
+    ) -> Response:
+        """Send a request with caching, rate-limiting, and retries
+
+        Args:
+            request: Prepared request to send
+            expire_after: How long to keep cached API requests
+            refresh: Skip reading from the cache and always fetch a fresh response
+            timeout: Maximum number of seconds to wait for a response from the server
+
+        **Note:** :py:meth:`requests.Session.send` accepts separate timeout values for connect and
+        read timeouts. The ``timeout`` argument will be used as the read timeout.
+        """
+        read_timeout = timeout or self.timeout
+        return super().send(
+            request,
+            expire_after=expire_after,
+            refresh=refresh,
+            timeout=(CONNECT_TIMEOUT, read_timeout),
+            **kwargs,
+        )
 
 
 def request(
     method: str,
     url: str,
     access_token: str = None,
+    allow_str_ids: bool = False,
     dry_run: bool = False,
+    expire_after: ExpirationTime = None,
     files: FileOrPath = None,
     headers: Dict = None,
     ids: MultiInt = None,
     json: Dict = None,
     raise_for_status: bool = True,
+    refresh: bool = False,
     session: Session = None,
+    timeout: int = None,
     **params: RequestParams,
 ) -> Response:
     """Wrapper around :py:func:`requests.request` with additional options specific to iNat API requests
@@ -152,10 +201,12 @@ def request(
         url: Request URL
         access_token: access_token: the access token, as returned by :func:`get_access_token()`
         dry_run: Just log the request instead of sending a real request
+        expire_after: How long to keep cached API requests
         files: File object, path, or URL to upload
         headers: Request headers
         ids: One or more integer IDs used as REST resource(s) to request
         json: JSON request body
+        refresh: Skip reading from the cache and always fetch a fresh response
         session: An existing Session object to use instead of creating a new one
         timeout: Time (in seconds) to wait for a response from the server; if exceeded, a
             :py:exc:`requests.exceptions.Timeout` will be raised.
@@ -170,6 +221,7 @@ def request(
         method=method,
         url=url,
         access_token=access_token,
+        allow_str_ids=allow_str_ids,
         files=files,
         headers=headers,
         ids=ids,
@@ -183,7 +235,13 @@ def request(
         return MOCK_RESPONSE
 
     # Otherwise, send the request
-    response = session.send(request)
+    session_kwargs = {'timeout': timeout}
+    if isinstance(session, CacheMixin):
+        session_kwargs['expire_after'] = expire_after
+        session_kwargs['refresh'] = refresh
+    response = session.send(request, **session_kwargs)
+    logger.debug(format_response(response))
+
     if raise_for_status:
         response.raise_for_status()
     return response
@@ -199,12 +257,13 @@ def prepare_request(
     ids: MultiInt = None,
     json: Dict = None,
     params: RequestParams = None,
+    allow_str_ids: bool = False,
     **kwargs,
 ) -> PreparedRequest:
     """Translate ``pyinaturalist``-specific options into standard request arguments"""
     # Prepare request params and URL
     params = preprocess_request_params(params)
-    url = convert_url_ids(url, ids)
+    url = convert_url_ids(url, ids, allow_str_ids)
 
     # Set auth header
     headers = headers or {}
@@ -253,6 +312,11 @@ def put(url: str, **kwargs) -> Response:
     return request('PUT', url, **kwargs)
 
 
+def clear_cache():
+    """Clear all cached API responses"""
+    get_local_session().cache.clear()
+
+
 def env_to_bool(environment_variable: str) -> bool:
     """Translate an environment variable to a boolean value, accounting for minor
     variations (case, None vs. False, etc.)
@@ -261,7 +325,7 @@ def env_to_bool(environment_variable: str) -> bool:
     return bool(env_value) and str(env_value).lower() not in ['false', 'none']
 
 
-def get_local_session(**kwargs) -> Session:
+def get_local_session(**kwargs) -> ClientSession:
     """Get a thread-local Session object with default settings. This will be reused across requests
     to take advantage of connection pooling and (optionally) caching. If used in a multi-threaded
     context (for example, a :py:class:`~concurrent.futures.ThreadPoolExecutor`), this will create
@@ -275,6 +339,29 @@ def get_local_session(**kwargs) -> Session:
     return thread_local.session
 
 
+def get_refresh_params(endpoint) -> Dict:
+    """In some cases, we need to be sure we have the most recent version of a resource, for example
+    when updating projects. Normally we would handle this with cache headers, but the CDN cache does
+    not respect these if the cached response is less than ~2 minutes old.
+
+    The iNat webapp handles this by adding an extra `v` request parameter, which results in a
+    different cache key. This function does the same by tracking a separate rate limit per value,
+    and finding the lowest value that is certain to result in a fresh response.
+    """
+    v = 0
+    while True:
+        try:
+            bucket = f'{endpoint}?v={v}'
+            REFRESH_LIMITER.try_acquire(bucket)
+            break
+        except BucketFullException as e:
+            seconds = int(e.meta_info["remaining_time"])
+            logger.debug(f'{bucket} cannot be refreshed again for {seconds} seconds')
+            v += 1
+
+    return {'refresh': True, 'v': v} if v > 0 else {'refresh': True}
+
+
 def is_dry_run_enabled(method: str) -> bool:
     """A wrapper to determine if dry-run (aka test mode) has been enabled via either
     a constant or an environment variable. Dry-run mode may be enabled for either write
