@@ -1,3 +1,4 @@
+# TODO: Move most of these test cases to controller tests
 """Things to test for each model:
 
 * Load with sample response JSON, make sure it doesn't explode
@@ -7,12 +8,21 @@
 * Any additional properties or aliases on the model
 * Formatting in the model's __str__ method
 """
+from copy import deepcopy
+from datetime import datetime
+
 # flake8: noqa: F405
 import pytest
-from datetime import datetime
 from dateutil.tz import tzoffset, tzutc
 
-from pyinaturalist.constants import ICONIC_TAXA, INAT_BASE_URL, PHOTO_INFO_BASE_URL, PHOTO_SIZES
+from pyinaturalist.constants import (
+    ICONIC_TAXA,
+    INAT_BASE_URL,
+    PHOTO_BASE_URL,
+    PHOTO_CC_BASE_URL,
+    PHOTO_INFO_BASE_URL,
+    PHOTO_SIZES,
+)
 from pyinaturalist.models import *
 from test.conftest import sample_data_path
 from test.sample_data import *
@@ -21,9 +31,10 @@ from test.sample_data import *
 # --------------------
 
 
-# TODO
 def test_from_json():
-    pass
+    obs = Observation.from_json(j_observation_1)
+    assert obs.id == 16227955
+    assert Observation.from_json(obs) is obs
 
 
 def test_from_json_file():
@@ -31,11 +42,47 @@ def test_from_json_file():
     assert isinstance(obs_list, list)
     assert isinstance(obs_list[0], Observation)
     assert obs_list[0].id == 57754375
+    assert Observation.from_json_file(None) == []
 
 
-# TODO
 def test_from_json_list():
-    pass
+    obs_list = Observation.from_json_list(SAMPLE_DATA['get_observations_node_page1'])
+    assert isinstance(obs_list, list)
+    assert isinstance(obs_list[0], Observation)
+    assert obs_list[0].id == 57754375
+
+
+def test_copy():
+    obs_1 = Observation.from_json(j_observation_1)
+    obs_2 = Observation.copy(obs_1)
+    assert obs_1.id == obs_2.id and obs_1.taxon == obs_2.taxon
+
+    # Make sure it's not a shallow copy
+    obs_2.taxon.id = 1234
+    assert obs_1.taxon.id != obs_2.taxon.id
+
+
+def test_deepcopy():
+    obs_1 = Observation.from_json(j_observation_1)
+    obs_2 = deepcopy(obs_1)
+    assert obs_1.id == obs_2.id and obs_1.taxon == obs_2.taxon
+    obs_2.taxon.id = 1234
+    assert obs_1.taxon.id != obs_2.taxon.id
+
+
+def test_copy_collection():
+    obs_list_1 = Observations.from_json([j_observation_1, j_observation_2])
+    obs_list_2 = Observations.copy(obs_list_1)
+    assert obs_list_1[0].id == obs_list_2[0].id and obs_list_1[0].taxon == obs_list_2[0].taxon
+    obs_list_2[0].taxon.id = 1234
+    assert obs_list_1[0].taxon.id != obs_list_2[0].taxon.id
+
+
+def test_deduplicate():
+    obs_list = Observations.from_json([j_observation_1, j_observation_1, j_observation_2])
+    assert len(obs_list) == 3
+    obs_list.deduplicate()
+    assert len(obs_list) == 2
 
 
 # Controlled Terms
@@ -79,7 +126,8 @@ def test_controlled_term__empty():
 def test_controlled_term__properties():
     controlled_term = ControlledTerm.from_json(j_controlled_term_1)
     assert (
-        controlled_term.value_labels == 'No Evidence of Flowering, Flowering, Fruiting, Flower Budding'
+        controlled_term.value_labels
+        == 'No Evidence of Flowering, Flowering, Fruiting, Flower Budding'
     )
 
 
@@ -114,7 +162,7 @@ def test_comment__empty():
 
 def test_comment__str():
     comment = Comment.from_json(j_comment_1)
-    assert str(comment) == 'samroom at 2020-08-28 12:04:18+00:00: Thankyou '
+    assert str(comment) == 'samroom on Aug 28, 2020: Thankyou'
 
 
 # Identifications
@@ -137,7 +185,7 @@ def test_identification__str():
     identification = ID.from_json(j_identification_3)
     assert str(identification) == (
         '[126501311] 🦋 Species: Danaus plexippus (Monarch) (improving) added on '
-        '2020-08-27 13:00:51-05:00 by samroom'
+        'Aug 27, 2020 by samroom'
     )
 
 
@@ -155,15 +203,37 @@ def test_life_list__converters():
 def test_life_list__empty():
     life_list = LifeList()
     life_list.data == []
-    life_list._taxon_counts is None
+    life_list._id_map is None
 
 
-def test_life_list__count():
+def test_life_list__get_count():
     life_list = LifeList.from_json(j_life_list)
-    assert life_list.count(1) == 3023  # Animalia
-    assert life_list.count(981) == 2  # Phasianus colchicus
-    assert life_list.count(-1) == 4  # Observations with no taxon
-    assert life_list.count(9999999) == 0  # Unobserved taxon
+    assert life_list.get_count(1) == 3023  # Animalia
+    assert life_list.get_count(981) == 2  # Phasianus colchicus
+    assert life_list.get_count(-1) == 4  # Observations with no taxon
+    assert life_list.get_count(9999999) == 0  # Unobserved taxon
+
+
+# Messages
+# --------------------
+
+
+def test_message__converters():
+    message = Message.from_json(j_message)
+    assert isinstance(message.to_user, User) and message.to_user.id == 2115051
+    assert isinstance(message.from_user, User) and message.from_user.id == 12345
+
+
+def test_message__empty():
+    message = Message()
+    assert message.thread_flags == []
+    assert message.to_user is None
+    assert message.from_user is None
+
+
+def test_message__str():
+    message = Message.from_json(j_message)
+    assert str(message) == '[12345] Sent Sep 02, 2019 from test_user to jkcook: Re: Test Message'
 
 
 # Observations
@@ -183,6 +253,7 @@ def test_observation__converters():
     assert isinstance(obs.user, User) and obs.user.id == 2852555
 
     assert obs.location == (50.0949055, -104.71929167)
+    assert obs.private_location == (50.0949055, -104.71929167)
 
 
 def test_observation__empty():
@@ -212,7 +283,9 @@ def test_observation__project_observations():
 
 def test_observation__thumbnail_url():
     obs = Observation.from_json(j_observation_2)
-    assert obs.thumbnail_url == 'https://static.inaturalist.org/photos/92152429/square.jpg?1598551272'
+    assert (
+        obs.thumbnail_url == 'https://static.inaturalist.org/photos/92152429/square.jpg?1598551272'
+    )
 
 
 def test_observations():
@@ -293,6 +366,20 @@ def test_photo__empty():
     assert photo.original_dimensions == (0, 0)
 
 
+def test_photo__guess_url():
+    photo = Photo(id=123)
+    assert photo.url == f'{PHOTO_BASE_URL}/123?size=original'
+    photo = Photo(id=123, license_code='CC-BY-NC')
+    assert photo.url == f'{PHOTO_CC_BASE_URL}/123/original.jpg'
+
+
+def test_photo__properties():
+    photo = Photo.from_json(j_photo_1)
+    assert photo.mimetype == 'image/jpeg'
+    assert photo.dimensions_str == '2048x1365'
+    assert photo.info_url == f'{PHOTO_INFO_BASE_URL}/38359335'
+
+
 def test_photo__license():
     photo = Photo.from_json(j_photo_1)
     assert photo.has_cc_license is True
@@ -352,19 +439,6 @@ def test_places__nearby():
 # --------------------
 
 
-def test_project__converters():
-    project = Project.from_json(j_project_1)
-    assert project.location == (48.777404, -122.306929)
-    assert project.project_observation_rules == project.obs_rules
-    assert project.obs_rules[0]['id'] == 616862
-    assert project.search_parameters[0]['field'] == 'quality_grade'
-    assert project.user_ids[-1] == 3387092 and len(project.user_ids) == 33
-
-    admin = project.admins[0]
-    assert isinstance(admin, ProjectUser) and admin.id == 233188 and admin.role == 'manager'
-    assert isinstance(project.user, User) and project.user.id == 233188
-
-
 def test_project__empty():
     project = Project()
     assert project.admins == []
@@ -375,15 +449,6 @@ def test_project__empty():
     assert project.user is None
 
 
-def test_project__with_obs_fields():
-    project = Project.from_json(j_project_3_obs_fields)
-    obs_field = project.project_observation_fields[0]
-    assert isinstance(obs_field, ProjectObservationField)
-    assert obs_field.id == 30
-    assert obs_field.position == 0
-    assert obs_field.required is False
-
-
 # Search
 # --------------------
 
@@ -431,15 +496,34 @@ def test_taxon__converters():
 def test_taxon__empty():
     taxon = Taxon()
     assert taxon.preferred_common_name == ''
+    assert taxon.iconic_taxon_name == 'Unknown'
     assert taxon.ancestors == []
     assert taxon.children == []
-    assert taxon.default_photo is None
     assert taxon.taxon_photos == []
 
 
+def test_taxon__str():
+    taxon_1 = Taxon(id=3, name='Aves', preferred_common_name='birb', rank='class')
+    assert str(taxon_1) == '[3] 🐦 Class: Aves (birb)'
+
+    taxon_2 = Taxon(id=3, name='Aves', rank='class')
+    assert str(taxon_2) == '[3] 🐦 Class: Aves'
+
+    taxon_3 = Taxon(id=3, name='Aves')
+    assert str(taxon_3) == '[3] 🐦 Aves'
+
+    taxon_4 = Taxon(id=0)
+    assert str(taxon_4) == '[0] ❓ unknown taxon'
+
+
 def test_taxon__all_names():
     taxon = Taxon.from_json(j_taxon_8_all_names)
-    assert taxon.names[1] == {'is_valid': True, 'name': 'American Crow', 'position': 0, 'locale': 'en'}
+    assert taxon.names[1] == {
+        'is_valid': True,
+        'name': 'American Crow',
+        'position': 0,
+        'locale': 'en',
+    }
 
 
 def test_taxon__autocomplete():
@@ -452,6 +536,17 @@ def test_taxon__conservation_status():
     assert isinstance(cs, ConservationStatus)
     assert cs.authority == 'NatureServe'
     assert cs.status_name == 'imperiled'
+    assert str(cs) == 'S2B (imperiled) via NatureServe'
+
+
+def test_taxon__conservation_status_aliases():
+    cs = Taxon.from_json(j_taxon_5_cs_status).conservation_status
+    cs.user_id = 111
+    cs.updater_id = 222
+    cs.place_id = 333
+    assert cs.user.id == cs.user_id == 111
+    assert cs.updater.id == cs.updater_id == 222
+    assert cs.place.id == cs.place_id == 333
 
 
 def test_taxon__conservation_statuses():
@@ -469,7 +564,16 @@ def test_taxon__establishment_means():
     assert es.establishment_means == str(es) == 'introduced'
 
 
-def test_taxon__taxonomy():
+def test_taxon__listed_taxa():
+    taxon = Taxon.from_json(j_taxon_1)
+    listed_taxon = taxon.listed_taxa[0]
+    assert isinstance(listed_taxon, ListedTaxon)
+    assert listed_taxon.taxon_id == taxon.id
+    assert taxon.listed_taxa_count == 4
+    assert str(listed_taxon) == '[70118] (native): 0 observations, 0 comments'
+
+
+def test_taxon__children_ancestors():
     taxon = Taxon.from_json(j_taxon_1)
     parent = taxon.ancestors[0]
     child = taxon.children[0]
@@ -497,12 +601,41 @@ def test_taxon__icon_url(taxon_name):
     assert taxon.icon_url is not None
 
 
+@pytest.mark.parametrize('taxon_id', ICONIC_TAXA.keys())
+def test_taxon__no_default_photo(taxon_id):
+    taxon = Taxon(iconic_taxon_id=taxon_id)
+    photo = taxon.default_photo
+    assert isinstance(photo, IconPhoto)
+    assert taxon.icon_url is not None
+    assert taxon.iconic_taxon_name is not None
+    assert photo.url is not None
+
+
 def test_taxon_properties__partial():
     taxon = Taxon.from_json(j_taxon_2_partial)
     assert taxon.ancestry.startswith('48460 | 1 | 47120 | ')
     assert taxon.parent is None
 
 
+def test_taxon__taxonomy():
+    taxon = Taxon.from_json(j_taxon_1)
+    assert taxon.taxonomy == {
+        'kingdom': 'Animalia',
+        'phylum': 'Arthropoda',
+        'subphylum': 'Hexapoda',
+        'class': 'Insecta',
+        'subclass': 'Pterygota',
+        'order': 'Coleoptera',
+        'suborder': 'Polyphaga',
+        'infraorder': 'Staphyliniformia',
+        'superfamily': 'Staphylinoidea',
+        'family': 'Silphidae',
+        'subfamily': 'Nicrophorinae',
+        'genus': 'Nicrophorus',
+        'species': 'Nicrophorus vespilloides',
+    }
+
+
 # TODO
 def test_taxon__update_from_full_record():
     pass
@@ -518,6 +651,7 @@ def test_taxon_counts__converters():
 def test_taxon_counts__empty():
     taxon_counts = TaxonCounts()
     assert taxon_counts.data == []
+    assert taxon_counts.id_map == {}
 
 
 # Users
@@ -550,3 +684,28 @@ def test_user__properties():
 def test_user__str():
     user = User.from_json(j_user_1)
     assert str(user) == '[1] kueda (Ken-ichi Ueda)'
+
+
+def test_user_count__str():
+    user_count = UserCount.from_json(j_observation_identifiers['results'][0])
+    assert str(user_count) == '[112514] earthstephen (Stephen): 1'
+
+
+def test_user_counts__identifiers():
+    user_counts = UserCounts.from_json(j_observation_identifiers)
+    assert user_counts.data[0] == user_counts[0]
+    assert len(user_counts) == 3
+    assert isinstance(user_counts.data[0], UserCount) and user_counts.data[0].count == 1
+
+
+def test_user_counts__observers():
+    user_counts = UserCounts.from_json(j_observation_observers)
+    assert user_counts.data[0] == user_counts[0]
+    assert len(user_counts) == 2
+    assert isinstance(user_counts.data[0], UserCount) and user_counts.data[0].count == 750
+
+
+def test_user_counts__empty():
+    user_counts = UserCounts()
+    assert user_counts.data == []
+    assert user_counts.id_map == {}
